/* GSequencer - Advanced GTK Sequencer
 * Copyright (C) 2019 Joël Krähemann
 *
 * This file is part of GSequencer.
 *
 * GSequencer is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * GSequencer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GSequencer.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glib.h>
#include <glib-object.h>

#include <ags/libags.h>

#include <CUnit/CUnit.h>
#include <CUnit/Automated.h>
#include <CUnit/Basic.h>

int ags_certificate_manager_test_init_suite();
int ags_certificate_manager_test_clean_suite();

void ags_certificate_manager_test_get_certificate();
void ags_certificate_manager_test_add_certificate();
void ags_certificate_manager_test_remove_certificate();

#define AGS_CERTIFICATE_MANAGER_TEST_GET_CERTIFICATE_XML_CERTIFICATE_COUNT (7)

#define AGS_CERTIFICATE_MANAGER_TEST_ADD_CERTIFICATE_XML_CERTIFICATE_COUNT (9)

#define AGS_CERTIFICATE_MANAGER_TEST_REMOVE_CERTIFICATE_XML_CERTIFICATE_COUNT (13)

AgsServerApplicationContext *server_application_context;

/* The suite initialization time.
 * Opens the temporary file used by the tests.
 * Returns zero on success, non-zero otherwise.
 */
int
ags_certificate_manager_test_init_suite()
{
  AgsConfig *config;

  ags_priority_load_defaults(ags_priority_get_instance());  

  config = ags_config_get_instance();

  server_application_context = (AgsApplicationContext *) ags_server_application_context_new();
  g_object_ref(server_application_context);

  ags_application_context_prepare(server_application_context);
  ags_application_context_setup(server_application_context);

  return(0);
}

/* The suite cleanup time.
 * Closes the temporary file used by the tests.
 * Returns zero on success, non-zero otherwise.
 */
int
ags_certificate_manager_test_clean_suite()
{
  return(0);
}

void
ags_certificate_manager_test_get_certificate()
{
  AgsCertificateManager *certificate_manager;
  AgsXmlCertificate **xml_certificate;

  GList *start_certificate, *certificate;

  guint i;
  
  certificate_manager = ags_certificate_manager_get_instance();

  certificate_manager->certificate = NULL;
  
  /* assert #0 - empty */
  start_certificate = ags_certificate_manager_get_certificate(certificate_manager);

  CU_ASSERT(start_certificate == NULL);

  /* assert #1 - with certificate */
  xml_certificate = (AgsXmlCertificate **) malloc(AGS_CERTIFICATE_MANAGER_TEST_GET_CERTIFICATE_XML_CERTIFICATE_COUNT * sizeof(AgsXmlCertificate *));

  for(i = 0; i < AGS_CERTIFICATE_MANAGER_TEST_GET_CERTIFICATE_XML_CERTIFICATE_COUNT; i++){
    xml_certificate[i] = ags_xml_certificate_new();
    certificate_manager->certificate = g_list_prepend(certificate_manager->certificate,
							    xml_certificate[i]);
  }
  
  start_certificate = ags_certificate_manager_get_certificate(certificate_manager);

  CU_ASSERT(start_certificate != NULL);
  CU_ASSERT(g_list_length(start_certificate) == AGS_CERTIFICATE_MANAGER_TEST_GET_CERTIFICATE_XML_CERTIFICATE_COUNT);
}

void
ags_certificate_manager_test_add_certificate()
{
  AgsCertificateManager *certificate_manager;
  AgsXmlCertificate **xml_certificate;

  GList *start_certificate, *certificate;

  guint i;
  
  certificate_manager = ags_certificate_manager_get_instance();

  certificate_manager->certificate = NULL;
  
  /* assert #0 - empty */
  start_certificate = ags_certificate_manager_get_certificate(certificate_manager);

  CU_ASSERT(start_certificate == NULL);

  /* assert #1 - with certificate */
  xml_certificate = (AgsXmlCertificate **) malloc(AGS_CERTIFICATE_MANAGER_TEST_ADD_CERTIFICATE_XML_CERTIFICATE_COUNT * sizeof(AgsXmlCertificate *));

  for(i = 0; i < AGS_CERTIFICATE_MANAGER_TEST_ADD_CERTIFICATE_XML_CERTIFICATE_COUNT; i++){
    xml_certificate[i] = ags_xml_certificate_new();
    ags_certificate_manager_add_certificate(certificate_manager,
						  xml_certificate[i]);
  }
  
  start_certificate = ags_certificate_manager_get_certificate(certificate_manager);

  CU_ASSERT(start_certificate != NULL);
  CU_ASSERT(g_list_length(start_certificate) == AGS_CERTIFICATE_MANAGER_TEST_ADD_CERTIFICATE_XML_CERTIFICATE_COUNT);
}

void
ags_certificate_manager_test_remove_certificate()
{
  AgsCertificateManager *certificate_manager;
  AgsXmlCertificate **xml_certificate;

  GList *start_certificate, *certificate;

  guint i;
  
  certificate_manager = ags_certificate_manager_get_instance();

  certificate_manager->certificate = NULL;
  
  /* assert #0 - empty */
  start_certificate = ags_certificate_manager_get_certificate(certificate_manager);

  CU_ASSERT(start_certificate == NULL);

  /* assert #1 - with certificate */
  xml_certificate = (AgsXmlCertificate **) malloc(AGS_CERTIFICATE_MANAGER_TEST_REMOVE_CERTIFICATE_XML_CERTIFICATE_COUNT * sizeof(AgsXmlCertificate *));

  for(i = 0; i < AGS_CERTIFICATE_MANAGER_TEST_REMOVE_CERTIFICATE_XML_CERTIFICATE_COUNT; i++){
    xml_certificate[i] = ags_xml_certificate_new();
    certificate_manager->certificate = g_list_prepend(certificate_manager->certificate,
							    xml_certificate[i]);
  }
  
  for(i = 0; i < AGS_CERTIFICATE_MANAGER_TEST_REMOVE_CERTIFICATE_XML_CERTIFICATE_COUNT; i++){
    ags_certificate_manager_remove_certificate(certificate_manager,
						     xml_certificate[i]);
  }

  start_certificate = ags_certificate_manager_get_certificate(certificate_manager);

  CU_ASSERT(start_certificate == NULL);
}

int
main(int argc, char **argv)
{
  CU_pSuite pSuite = NULL;
  
  /* initialize the CUnit test registry */
  if(CUE_SUCCESS != CU_initialize_registry()){
    return CU_get_error();
  }

  /* add a suite to the registry */
  pSuite = CU_add_suite("AgsCertificateManagerTest", ags_certificate_manager_test_init_suite, ags_certificate_manager_test_clean_suite);
  
  if(pSuite == NULL){
    CU_cleanup_registry();
    
    return CU_get_error();
  }

  /* add the tests to the suite */
  if((CU_add_test(pSuite, "test of AgsCertificateManager get certificate", ags_certificate_manager_test_get_certificate) == NULL) ||
     (CU_add_test(pSuite, "test of AgsCertificateManager add certificate", ags_certificate_manager_test_add_certificate) == NULL) ||
     (CU_add_test(pSuite, "test of AgsCertificateManager remove certificate", ags_certificate_manager_test_remove_certificate) == NULL)){
    CU_cleanup_registry();
    
    return CU_get_error();
  }
  
  /* Run all tests using the CUnit Basic interface */
  CU_basic_set_mode(CU_BRM_VERBOSE);
  CU_basic_run_tests();
  
  CU_cleanup_registry();
  
  return(CU_get_error());
}
